<!-- Load jQuery JavaScript module (for Datatables) -->
<script src="https://code.jquery.com/jquery-3.5.1.slim.min.js"
  integrity="sha384-DfXdz2htPH0lsSSs5nCTpuj/zy4C+OGpamoFVy38MVBnE+IbbVYUew+OrCXaRkfj" crossorigin="anonymous"></script>


<!-- Load Popper JavaScript module -->
<script src="https://cdn.jsdelivr.net/npm/@popperjs/core@2.9.2/dist/umd/popper.min.js"
  integrity="sha384-IQsoLXl5PILFhosVNubq5LC7Qb9DXgDA9i+tQ8Zj3iwWAwPtgFTxbJ8NT4GN1R8p" crossorigin="anonymous"></script>

<!-- Load Bootstrap JavaScript module -->
<script src="https://cdn.jsdelivr.net/npm/bootstrap@5.0.2/dist/js/bootstrap.min.js"
  integrity="sha384-cVKIPhGWiC2Al4u+LWgxfKTRIcfu0JTxR+EQDz/bgldoEyl4H0zUF0QKbrJ0EcQF" crossorigin="anonymous"></script>

<!-- Load Font Awesome icons 
<script src="https://stackpath.bootstrapcdn.com/font-awesome/4.7.0/css/font-awesome.min.css"></script> -->

<!-- Load Bootstrap-Select CSS and JavaScript -->
<!--
<link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/bootstrap-select@1.13.14/dist/css/bootstrap-select.min.css">
<script src="https://cdn.jsdelivr.net/npm/bootstrap-select@1.13.14/dist/js/bootstrap-select.min.js"></script>
-->

<!-- This script removes extra fields from API forms on GET method. 
  Code is from https://stackoverflow.com/questions/5904437/
  Where I found this link from https://stackoverflow.com/questions/46834539/
  -->
<!-- The other option here is to switch to a post method via https://stackoverflow.com/questions/46834539/ -->
<script>
  $(function () {
    var form = $('form')[0];
    $(form).submit(function () {
      $('input, select').each(function () {
        if ($(this).val().length === 0) {
          $(this).prop('disabled', true);
          $(this).next().prop('disabled', true);
        }
      });
    });
  });
</script>