{% extends "core/site-base.html" %}

{% block tabtitle %}
Simmate
{% endblock %}

{% block banner %}
{% include "core/header.html"%}
{% endblock %}


{% block body %}

<div class="container p-4 bg-light border" id="search-results">

  <div class="container p-4">

    <!-- Breadcrumb links -->
    <nav aria-label="breadcrumb">
      <ol class="breadcrumb">
        <li class="breadcrumb-item"><a href="{% url 'workflows' %}">workflows</a></li>
        <li class="breadcrumb-item"><a href="{% url 'workflows_by_type' workflow.type %}">{{ workflow.type }}</a></li>
        <li class="breadcrumb-item"><a href="{% url 'workflow_detail' workflow.type workflow.name_short %}">
            {{ workflow.name_short }}</a></li>
        <li class="breadcrumb-item active">{{ calculation.id }}</li>
      </ol>
    </nav>
    
    <!-- Build views for the table entry's data -->
    {% include "core_components/table_entry.html" %}

  </div>

</div>

{% endblock %}
