<!-- Indicate which Character Encoding set the browser should use -->
<meta charset="utf-8">

<!-- Set the icons to use in the tab header -->
<link rel="apple-touch-icon" sizes="76x76" href="/static/images/simmate-icon.svg">
<link rel="icon" type="image/png" href="/static/images/simmate-icon.svg">

<!-- Set meta data for responsive viewport scaling/size (i.e. phone vs monitor) -->
<meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no">

<!-- Load Bootstrap CSS module -->
<link href="https://cdn.jsdelivr.net/npm/bootstrap@5.0.2/dist/css/bootstrap.min.css" rel="stylesheet"
    integrity="sha384-EVSTQN3/azprG1Anm3QDgpJLIm9Nao0Yz1ztcQTwFspd3yD65VohhpuuCOmLASjC" crossorigin="anonymous">

<!-- Load Bootstrap Icons -->
<link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/bootstrap-icons@1.5.0/font/bootstrap-icons.css">

<!-- Load Plotly Javascript (this is needed in the header, not the footer!)-->
<script src="https://cdn.plot.ly/plotly-latest.min.js"></script>

<!-- I don't use this now, but the links below are for custom css. This may be important for styling the site -->
<!-- <link rel="stylesheet" href="./scss/custom.css"> -->
<!-- {% load static %}
<link rel="stylesheet" type="text/css" href="{% static 'structures/style.css' %}"> -->