<!-- This code would go in the body -->
<label for="customRange1" class="form-label">Example range</label>
<input type="range" class="form-range slider" id="customRange1" min="0" max="5" step="0.1">
<p>Value: <span id="outValue1"></span></p>


<!-- This code would go in the footer -->
<script>
  var slider = document.getElementById("customRange1");
  var output = document.getElementById("outValue1");
  output.innerHTML = slider.value;

  slider.oninput = function () {
    output.innerHTML = this.value;
  }
</script>